/***********************************************************//**
 *
 * Implementation of Peak
 * @author Shinichi Fujino
 * @since 2022.4.1
 * @version 1.0.1.0
 */

///////////////////////////////////////////////////////
// include
#include <stdint.h>
#include "peak.h"

///////////////////////////////////////////////////////
// define


/***********************************************************//**
 *
 * @brief Initialization
 */

void Peak_reset( Peak_Obj* thisp, Peak_Params* params )
{
	thisp->adc_max = 0;
	
	return;
}


/***********************************************************//**
 *
 * @brief Set data
 * @param adc_u12 ADC data. 0~4095
 * @param count Count of ADC data
 */

void Peak_set( Peak_Obj* thisp, const uint16_t* adc_u12, int count )
{
	int i;
	int16_t adc_max = thisp->adc_max;
	
	for( i = 0; i < count; i++ )
	{
		// unsigned -> signed
		int16_t adc_s12 = (int16_t)adc_u12[i] - 2048;
		
		// abs
		if( adc_s12 < 0 ) adc_s12 = - adc_s12;
		
		// max
		if( adc_max < adc_s12 ) adc_max = adc_s12;
	}
	
	thisp->adc_max = adc_max;
	
	return;
}


/***********************************************************//**
 *
 * @brief Get data & Clear max value
 * @return max data
 */

int16_t Peak_get( Peak_Obj* thisp )
{
	int16_t adc_max = thisp->adc_max;
	
	thisp->adc_max = 0;
	
	return adc_max;
}


/*
 * Local Variables:
 *  tab-width : 4
 * End:
 *
 * EOF */
