﻿/***********************************************************//**
 *
 * Implementation of DownFilt
 * @author Shinichi Fujino
 * @since 2022.4.1
 * @version 1.0.1.0
 */

///////////////////////////////////////////////////////
// include
#include <stdint.h>
#include "jihoudet_def.h"
#include "downfilt.h"
#include "memset.h"
#include "memcpy.h"

///////////////////////////////////////////////////////
// define


/***********************************************************//**
 *
 * @var DownFilt_coeff75
 * @brief LPF coefficient
 * Filter length： 74
 * Sampling frequency： 6 kHz
 * Passband
 *   Lower limit frequency (kHz): 0.000
 *   Upper limit frequency (kHz): 1.320
 *   Amplitude                  : 1.0
 *   Weight                     : 1.0
 * Block bandwidth
 *   Lower limit frequency (kHz): 1.680
 *   Upper limit frequency (kHz): 3.000
 *   Amplitude                  : 0.0
 *   Weight                     : 1.0
 */

const float DownFilt_coeff75[DOWNFILT_TAP] =
{
	(float)1.52652985e-04, (float)1.03136877e-10, (float)-2.64029695e-04, (float)-6.59712401e-11,
	(float)4.81486271e-04, (float)2.29673438e-10, (float)-8.05611938e-04, (float)-2.42481832e-10,
	(float)1.26837498e-03, (float)4.80784890e-10, (float)-1.90723754e-03, (float)-5.39791371e-10,
	(float)2.76608638e-03, (float)8.39075753e-10, (float)-3.89694945e-03, (float)-9.45939614e-10,
	(float)5.36314991e-03, (float)1.28221212e-09, (float)-7.24508322e-03, (float)-1.43245311e-09,
	(float)9.65084796e-03, (float)1.77383500e-09, (float)-1.27362888e-02, (float)-1.93626177e-09,
	(float)1.67445101e-02, (float)2.28809981e-09, (float)-2.20894679e-02, (float)-2.40801185e-09,
	(float)2.95518560e-02, (float)2.70319348e-09, (float)-4.08113455e-02, (float)-2.76711297e-09,
	(float)6.02560850e-02, (float)2.97715948e-09, (float)-1.04028579e-01, (float)-2.96142433e-09,
	(float)3.17613048e-01, (float)5.00000003e-01, (float)3.17613048e-01, (float)-2.96142433e-09,
	(float)-1.04028579e-01, (float)2.97715948e-09, (float)6.02560850e-02, (float)-2.76711297e-09,
	(float)-4.08113455e-02, (float)2.70319348e-09, (float)2.95518560e-02, (float)-2.40801185e-09,
	(float)-2.20894679e-02, (float)2.28809981e-09, (float)1.67445101e-02, (float)-1.93626177e-09,
	(float)-1.27362888e-02, (float)1.77383500e-09, (float)9.65084796e-03, (float)-1.43245311e-09,
	(float)-7.24508322e-03, (float)1.28221212e-09, (float)5.36314991e-03, (float)-9.45939614e-10,
	(float)-3.89694945e-03, (float)8.39075753e-10, (float)2.76608638e-03, (float)-5.39791371e-10,
	(float)-1.90723754e-03, (float)4.80784890e-10, (float)1.26837498e-03, (float)-2.42481832e-10,
	(float)-8.05611938e-04, (float)2.29673438e-10, (float)4.81486271e-04, (float)-6.59712401e-11,
	(float)-2.64029695e-04, (float)1.03136877e-10, (float)1.52652985e-04,
};


/***********************************************************//**
 *
 * @brief Initialization
 * @param thisp Object pointer
 * @param params Parameter pointer
 */

void DownFilt_reset( DownFilt_Obj* thisp, const DownFilt_Params* params )
{
	// TODO:Write the initialization code in this position
	memset_f32( thisp->m_mem, 0, sizeof(thisp->m_mem) / sizeof(float) );
	
	return;
}


/***********************************************************//**
 *
 * @brief Convolution
 * @param thisp Object pointer
 * @param src Input data pointer
 * @param dst Output data pointer
 * @param length Data length
 */

static void DownFilt_conv( DownFilt_Obj* thisp, const float* src, float* dst, int lenDst )
{
	// TODO:Write the code for convolution in this position
	int k;
	
	for( k = 0; k < lenDst; k++ )
	{
		int i;
		float acc = 0;
		
		for( i = 0; i < DOWNFILT_TAP; i++ )
		{
			acc = acc + ( src[2 * k + i] * DownFilt_coeff75[i] );
		}
		
		dst[k] = acc;
	}
	
	return;
}


/***********************************************************//**
 *
 * @brief Execution
 * @param thisp Object pointer
 * @param src Input data pointer
 * @param dst Output data pointer
 */

void DownFilt_apply( DownFilt_Obj* thisp, const float src[JIHOUDET_FRAME_12K], float dst[JIHOUDET_FRAME_06K] )
{
	// TODO:Write the code for execution in this position
	float scratch[JIHOUDET_FRAME_12K + DOWNFILT_TAP];
	
	// Load
	memcpy_f32( scratch, thisp->m_mem, DOWNFILT_TAP );
	
	memcpy_f32( &scratch[DOWNFILT_TAP], src, JIHOUDET_FRAME_12K );
	
	// Store
	memcpy_f32( thisp->m_mem, &scratch[JIHOUDET_FRAME_12K], DOWNFILT_TAP );
	
	// Convolution
	DownFilt_conv( thisp, scratch, dst, JIHOUDET_FRAME_06K );
	
	return;
}


/*
 * Local Variables:
 *  tab-width : 4
 * End:
 *
 * EOF */
