﻿/***********************************************************//**
 *
 * Implementation of CJihouDet
 * @author Shinichi Fujino
 * @since 2022.4.1
 * @version 1.0.1.0
 */

///////////////////////////////////////////////////////
// include
#include <stdint.h>
#include "stdbool.h"
#include "jihoudet_def.h"
#include "jihoudet_api.h"
#include "memset.h"
#include "memcpy.h"
#include "corr.h"
#include "corr_table.h"
#include "search.h"

///////////////////////////////////////////////////////
// define



/***********************************************************//**
 *
 * @brief Initialization
 * @param thisp Object pointer
 * @param params Parameter pointer
 */

void CJihouDet_reset( CJihouDet_Obj* thisp, const CJihouDet_Params* params )
{
	// TODO:Write the initialization code in this position
	// Downsampling
	DownFilt_reset( &thisp->downfilt, &params->downfilt_prm );
	
	// DC Cut
	DcCut_reset( &thisp->dccut, &params->dccut_prm );
	
	// Judgement
	Judge_reset( &thisp->judge, &params->judge_prm );
	
	// Previous data
	memset_f32( thisp->m_mem, 0, sizeof(thisp->m_mem) / sizeof(float) );
	
	return;
}


/***********************************************************//**
 *
 * @brief Execution
 * @param thisp Object pointer
 * @param src Input data pointer
 * @retval false No-Detected
 * @retval true Detected
 */

bool CJihouDet_apply( CJihouDet_Obj* thisp, const float src[JIHOUDET_FRAME_12K] )
{
	// TODO:Write the code for execution in this position
	float tone6k[JIHOUDET_FRAME_06K];
	float dccut6k[JIHOUDET_FRAME_CONV];
	float corr_pow[JIHOUDET_SEARCH_NUM];
	int idx;
	bool detect;
	
	// Downsampling
	DownFilt_apply( &thisp->downfilt, src, tone6k );
	
	// DC Cut
	DcCut_apply( &thisp->dccut, tone6k, &dccut6k[1 * JIHOUDET_FRAME_06K] );
	
	//// Save 2 frames
	// Loading the previous frame
	memcpy_f32( &dccut6k[0 * JIHOUDET_FRAME_06K], thisp->m_mem, JIHOUDET_FRAME_06K );
	
	// Loading the current frame
	memcpy_f32( thisp->m_mem, &dccut6k[1 * JIHOUDET_FRAME_06K], JIHOUDET_FRAME_06K );
	
	// Window & Correlation. Coefficient multiplied by hann window
	corr_pow[0] = Correlation( dccut6k, Corr_hann440Hz );
	corr_pow[1] = Correlation( dccut6k, Corr_hann880Hz );
	
	corr_pow[2] = Correlation( dccut6k, Corr_hann100Hz );
	corr_pow[3] = Correlation( dccut6k, Corr_hann200Hz );
	corr_pow[4] = Correlation( dccut6k, Corr_hann300Hz );
	corr_pow[5] = Correlation( dccut6k, Corr_hann400Hz );
	corr_pow[6] = Correlation( dccut6k, Corr_hann500Hz );
	corr_pow[7] = Correlation( dccut6k, Corr_hann600Hz );
	corr_pow[8] = Correlation( dccut6k, Corr_hann700Hz );
	corr_pow[9] = Correlation( dccut6k, Corr_hann800Hz );
	corr_pow[10] = Correlation( dccut6k, Corr_hann1000Hz );
	
	// Search for maximum value
	idx = Search( corr_pow );
	
	// Judgement
	detect = Judge_apply( &thisp->judge, corr_pow, idx );
	
	return detect;
}


/*
 * Local Variables:
 *  tab-width : 4
 * End:
 *
 * EOF */
