﻿/***********************************************************//**
 *
 * Interface of CJihouDet
 * @author Shinichi Fujino
 * @since 2022.4.1
 * @version 1.0.1.0
 */

#ifndef __JIHOUDET_H__INCLUDED__
#define __JIHOUDET_H__INCLUDED__

#include "downfilt.h"
#include "dccut.h"
#include "judge.h"

/***********************************************************//**
 *
 * @struct CJihouDet_Params
 * @brief Initialization parameters
 *
 */

typedef struct
{
	DownFilt_Params downfilt_prm; /**< Downsampling */
	DcCut_Params dccut_prm; /**< DC Cut */
	Judge_Params judge_prm; /**< Judgement */
} CJihouDet_Params;


/***********************************************************//**
 *
 * @struct CJihouDet_Obj
 * @brief Object
 *
 */

typedef struct
{
	DownFilt_Obj downfilt; /**< Downsampling */
	DcCut_Obj dccut; /**< DC Cut */
	Judge_Obj judge; /**< Judgement */
	float m_mem[JIHOUDET_FRAME_06K]; /**< Previous data */
} CJihouDet_Obj;


#if defined __cplusplus
extern "C" {
#endif

extern void CJihouDet_reset( CJihouDet_Obj* thisp, const CJihouDet_Params* params );

extern bool CJihouDet_apply( CJihouDet_Obj* thisp, const float src[JIHOUDET_FRAME_12K] );

#if defined __cplusplus
}
#endif



#endif // __JIHOUDET_H__INCLUDED__


/*
 * Local Variables:
 *  tab-width : 4
 * End:
 *
 * EOF */
