﻿/***********************************************************//**
 *
 * Implementation of Judge
 * @author Shinichi Fujino
 * @since 2022.4.1
 * @version 1.0.1.0
 */

///////////////////////////////////////////////////////
// include
#include <stdint.h>
#include "stdbool.h"
#include "jihoudet_def.h"
#include "judge.h"
#include "memset.h"

///////////////////////////////////////////////////////
// define
#define IDX_CONV_POW_440HZ	(0) /**< Index of Correlation power 440[Hz] */
#define IDX_CONV_POW_880HZ	(1) /**< Index of Correlation power 880[Hz] */




/***********************************************************//**
 *
 * @brief Clearing the internal state
 * @param thisp Object pointer
 */

static void Judge_clear( Judge_Obj* thisp )
{
	// TODO:Write the initialization code in this position
	thisp->m_state = JUDGE_STATE_FORECAST1;
	
	thisp->m_count_silent = 0;
	
	memset_i32( thisp->m_hist_idx, -1, sizeof(thisp->m_hist_idx) / sizeof(int32_t) );
	
	return;
}


/***********************************************************//**
 *
 * @brief Initialization
 * @param thisp Object pointer
 * @param params Parameter pointer
 */

void Judge_reset( Judge_Obj* thisp, const Judge_Params* params )
{
	// TODO:この位置に初期化用のコードを記述してください
	thisp->m_thres_det = params->thres_det;
	
	Judge_clear( thisp );
	
	return;
}


/***********************************************************//**
 *
 * @brief History update
 * @param thisp Object pointer
 * @param corr_pow Correlation power pointer
 * @param idx_max Index of maximum correlation power
 */

static void Judge_hist( Judge_Obj* thisp, const float corr_pow[JIHOUDET_SEARCH_NUM], int idx_max )
{
	// TODO:Please write the code for updating the history in this position
	int i;
	
	for( i = JUDGE_HIST_NUM - 2; i >= 0; i-- )
	{
		thisp->m_hist_idx[i + 1] = thisp->m_hist_idx[i];
	}
	
	// Comparison with threshold
	if( corr_pow[idx_max] < thisp->m_thres_det )
	{
		idx_max |= 0x80000000; // Negative when lower than threshold
	}
	
	thisp->m_hist_idx[0] = idx_max;
	
	return;
}


/***********************************************************//**
 *
 * @brief Forecast sound 1st judgment
 * @param thisp Object pointer
 */

static void Judge_forecast1( Judge_Obj* thisp )
{
	// TODO:Please write the code for judgment in this position
	int i;
	int count = 0;
	
	for( i = 0; i < 10; i++ )
	{
		if( thisp->m_hist_idx[i] == IDX_CONV_POW_440HZ ) count += 1;
	}
	
	if( count >= 8 )
	{
		thisp->m_state = JUDGE_STATE_FORECAST2;
		
		thisp->m_count_silent = 0;
	}
	
	return;
}


/***********************************************************//**
 *
 * @brief Forecast sound 2nd judgment
 * @param thisp Object pointer
 */

static void Judge_forecast2( Judge_Obj* thisp )
{
	// TODO:Please write the code for judgment in this position
	
	thisp->m_count_silent += 1;
	
	if( thisp->m_count_silent >= 10 )
	{
		int i;
		int count = 0;
		
		for( i = 0; i < 10; i++ )
		{
			if( thisp->m_hist_idx[i] == IDX_CONV_POW_440HZ ) count += 1;
		}
		
		if( count >= 8 )
		{
			if( thisp->m_count_silent >= 89 )
			{
				// 予報音1回目から89フレーム以上経過している
				thisp->m_state = JUDGE_STATE_FORECAST3;
				
				thisp->m_count_silent = 0;
			}
			else
			{
				// 予報音1回目判定中へ戻る
				Judge_clear( thisp );
			}
		}
		else
		{
			if( thisp->m_count_silent >= 98 )
			{
				// 98フレーム以上予報音がなければ予報音1回目判定中へ戻る
				Judge_clear( thisp );
			}
		}
	}
	
	return;
}


/***********************************************************//**
 *
 * @brief Forecast sound 3rd judgment
 * @param thisp Object pointer
 */

static void Judge_forecast3( Judge_Obj* thisp )
{
	// TODO:Please write the code for judgment in this position
	
	thisp->m_count_silent += 1;
	
	if( thisp->m_count_silent >= 10 )
	{
		int i;
		int count = 0;
		
		for( i = 0; i < 10; i++ )
		{
			if( thisp->m_hist_idx[i] == IDX_CONV_POW_440HZ ) count += 1;
		}
		
		if( count >= 8 )
		{
			if( thisp->m_count_silent >= 89 )
			{
				// 予報音2回目から89フレーム以上経過している
				thisp->m_state = JUDGE_STATE_POSITIVE;
				
				thisp->m_count_silent = 0;
			}
			else
			{
				// 予報音1回目判定中へ戻る
				Judge_clear( thisp );
			}
		}
		else
		{
			if( thisp->m_count_silent >= 98 )
			{
				// 98フレーム以上予報音がなければ予報音1回目判定中へ戻る
				Judge_clear( thisp );
			}
		}
	}
	
	return;
}


/***********************************************************//**
 *
 * @brief Positive sound judgment
 * @param thisp Object pointer
 * @retval false No-Detected
 * @retval true Detected
 */

static bool cjudge_positive( Judge_Obj* thisp )
{
	// TODO:Please write the code for judgment in this position
	bool detect = false;
	
	thisp->m_count_silent += 1;
	
	if( thisp->m_count_silent >= 10 )
	{
		int i;
		int count = 0;
		
		for( i = 0; i < 100; i++ )
		{
			if( thisp->m_hist_idx[i] == IDX_CONV_POW_880HZ ) count += 1;
		}
		
		if( count >= 90 )
		{
			if( thisp->m_count_silent >= 173 )
			{
				// 予報音3回目から173フレーム以上経過している
				thisp->m_state = JUDGE_STATE_DETECT;
				
				thisp->m_count_silent = 0;
				
				detect = true;
			}
			else
			{
				// 予報音1回目判定中へ戻る
				Judge_clear( thisp );
			}
		}
		else
		{
			if( thisp->m_count_silent >= 183 )
			{
				// 183フレーム以上予報音がなければ予報音1回目判定中へ戻る
				Judge_clear( thisp );
			}
		}
	}
	
	return detect;
}


/***********************************************************//**
 *
 * @brief After detecting the positive sound
 * @param thisp Object pointer
 */

static void Judge_detect( Judge_Obj* thisp )
{
	// TODO:Write the code for after detecting the positive sound at this position.
	
	thisp->m_count_silent += 1;
	
	if( thisp->m_count_silent >= 200 )
	{
		Judge_clear( thisp );
	}
	
	return;
}


/***********************************************************//**
 *
 * @brief 実行
 *  Forecast sound 1st judgment
 *    Maximum value is greater than "Threshold 1"
 *    "Maximum correlation with 440[Hz]" is 8 frames or more out of 10 frames
 * 
 *  Forecast sound 2nd judgment
 *    Maximum value is greater than "Threshold 1"
 *    "Maximum correlation with 440[Hz]" is 8 frames or more out of 10 frames
 *    & Detected within 82-88 frames from Forecast sound 1st judgment
 * 
 *  Forecast sound 3rd judgment
 *    Maximum value is greater than "Threshold 1"
 *    "Maximum correlation with 440[Hz]" is 8 frames or more out of 10 frames
 *    & Detected within 82-88 frames from Forecast sound 2nd judgment
 * 
 *  Judgment of positive sound
 *    Maximum value is greater than "Threshold 1"
 *    "Maximum correlation with 880[Hz]" is 8 frames or more out of 10 frames
 *    & Detected within 82-88 frames from Forecast sound 3rd judgment
 * 
 *  After detecting the positive sound
 *    wait 200 frames
 * 
 * @param thisp Object pointer
 * @param corr_pow Correlation power pointer
 * @param idx_max Index of maximum correlation power
 * @retval false No-Detected
 * @retval true Detected
 */

bool Judge_apply( Judge_Obj* thisp, const float corr_pow[JIHOUDET_SEARCH_NUM], int idx_max )
{
	// TODO:Write the code for execution in this position
	bool detect = false;
	
	// History update
	Judge_hist( thisp, corr_pow, idx_max );
	
	if( thisp->m_state & JUDGE_STATE_FORECAST1 )
	{
		// Forecast sound 1st judgment in progress
		Judge_forecast1( thisp );
	}
	else if( thisp->m_state & JUDGE_STATE_FORECAST2 )
	{
		// Forecast sound 2nd judgment in progress
		Judge_forecast2( thisp );
	}
	else if( thisp->m_state & JUDGE_STATE_FORECAST3 )
	{
		// Forecast sound 3rd judgment in progress
		Judge_forecast3( thisp );
	}
	else if( thisp->m_state & JUDGE_STATE_POSITIVE )
	{
		// During judgment of positive sound
		detect = cjudge_positive( thisp );
	}
	else if( thisp->m_state & JUDGE_STATE_DETECT )
	{
		// After detecting the positive sound
		Judge_detect( thisp );
	}
	
	return detect;
}


/*
 * Local Variables:
 *  tab-width : 4
 * End:
 *
 * EOF */
