﻿/***********************************************************//**
 *
 * Interface of Judge
 * @author Shinichi Fujino
 * @since 2022.4.1
 * @version 1.0.1.0
 */

#ifndef __JUDGE_H__INCLUDED__
#define __JUDGE_H__INCLUDED__

#define JUDGE_HIST_NUM	(128) /**< Number of histories */

/***********************************************************//**
 *
 * @struct Judge_Params
 * @brief Initialization parameters
 *
 */

typedef struct
{
	float thres_det; /**< Sound threshold */
} Judge_Params;


/***********************************************************//**
 *
 * @enum Judge_Obj
 * @brief Status
 *
 */

typedef enum
{
	JUDGE_STATE_FORECAST1 = 0x0001, /**< Forecast sound 1st judgment in progress */
	JUDGE_STATE_FORECAST2 = 0x0002, /**< Forecast sound 2nd judgment in progress */
	JUDGE_STATE_FORECAST3 = 0x0004, /**< Forecast sound 3rd judgment in progress */
	JUDGE_STATE_POSITIVE = 0x0008, /**< During judgment of positive sound */
	JUDGE_STATE_DETECT = 0x0010 /**< Positive sound detection */
} Judge_State;


/***********************************************************//**
 *
 * @struct Judge_Obj
 * @brief Object
 *
 */

typedef struct
{
	float m_thres_det; /**< Sound threshold */
	
	Judge_State m_state; /**< Status */
	int32_t m_count_silent; /**< Number of silences */
	int32_t m_hist_idx[JUDGE_HIST_NUM]; /**< Index history. (new)[0]~[127](old) */
} Judge_Obj;


#if defined __cplusplus
extern "C" {
#endif

extern void Judge_reset( Judge_Obj* thisp, const Judge_Params* params );

extern bool Judge_apply( Judge_Obj* thisp, const float corr_pow[JIHOUDET_SEARCH_NUM], int idx_max );

#if defined __cplusplus
}
#endif



#endif // __JUDGE_H__INCLUDED__


/*
 * Local Variables:
 *  tab-width : 4
 * End:
 *
 * EOF */
